//******************************************************************************
// E-Z Reader, ver. 10.3; (c) 2021 Erik D. Reichle, Ph.D.
//
//      Notes: 
//
//      (1) 9/12/2021:   
//              This program implements the E-Z Reader model of eye-movement control during reading
//              (Reichle et al., 2012). The program uses word-frequency values from the ELP (Balota,
//              et al., 2007).
//      (2) 4/27/2022:
//              The program was compiled and tested using Java 17.0.2 running on the Apache Netbeans
//              IDE 12.6.
//
//******************************************************************************

package ezreader10;

import java.io.*;
import java.util.*;

public class EZReader10 {
 
    // I/O file names:
    static String corpusFile = "SRC98-ELP"; // name of sentence corpus file   
    static String outputFile = "SimulationResults"; // name of output file
    static String targetFile = "SRC98Targets"; // name of word target file

    // Model free parameters:
    static double A = 25.0; 
    static double Alpha1 = 127.0; 
    static double Alpha2 = 8.0; 
    static double Alpha3 = 59.0; 
    static double Delta = 1.21; 
    static double Epsilon = 1.05;
    static double Eta1 = 0.5;
    static double Eta2 = 0.1; 
    static double I = 50.0; 
    static double ITarget = 50.0;
    static double Lambda = 0.25; 
    static double M1 = 150.0;
    static double M2 = 25.0; 
    static double Omega1 = 6.0; 
    static double Omega2 = 3.0; 
    static double pF = 0.01; 
    static double pFTarget = 0.01;
    static double Psi = 7.0; 
    static double S = 25.0; 
    static double SigmaGamma = 20.0; 
    static double V = 50.0; 
    static double Xi = 0.5;
    
    // Fixed simulation parameters:
    static int maxLength = 16; // = # letters + space left of word
    static int maxSentenceLength = 50; // maximum # words / sentence
    static int NFreqClass = 8;
    static int NSentences = 48;
    static int NSubjects = 1000;
    
    // Display toggles:
    static boolean displayClassMeans = false; // frequency-class means & RMSD
    static boolean displayCorpus = false; // word IVs
    static boolean displayDists = false; // IOVP distributions, etc.
    static boolean displayMeans = true; // mean word DVs
    static boolean displayParameters = true; // parameter values
    static boolean displayTrace = false; // trace of individual fixations
    
    // Simulation toggles:
    static boolean includeRegressions = false; // include regressions?   
    
    // Statistical variables:
    static double bestRMSD;
    static double RMSD;
    
    //**************************************************************************
    
    public static void main(String[] args) 
            throws FileNotFoundException {
        
        // Initialize classes:
        Display display = new Display();
        EZReader ezreader = new EZReader();
        Stats stats = new Stats();
        
        // Initialize sentence corpus:
        Scanner diskScanner1 = new Scanner(new File(corpusFile + ".txt")); 
        ArrayList<Sentence> text = new ArrayList<>();
        Corpus corpus = new Corpus();
        corpus.initialize(diskScanner1, text);
        System.out.printf(" Corpus initialized.\n");
        
        // Initialize target words:
        Scanner diskScanner2 = new Scanner(new File(targetFile + ".txt"));
        int k = 0;
        while (diskScanner2.hasNext()) {
            text.get(k).target = diskScanner2.nextInt();
            k++;
        }
        System.out.printf(" Targets initialized.\n");
        
        // Initialize output file:
        PrintStream diskWriter = new PrintStream(new File(outputFile + ".txt")); 
        System.out.printf(" Output file initialized.\n\n");
        
        // Display model parameters and properties of words in sentence corpus:
        if (EZReader10.displayParameters == true) display.parameters(diskWriter);
        if (displayCorpus == true) display.corpus(diskWriter, text);
                
        // Search parameter space:
        int run = 0;
        bestRMSD = 10.0;
        double var1, var2, var3, var4; 
        for (var1 = 0; var1 < 0.01; var1 += 1.0) { 
            for (var2 = 0; var2 < 0.01; var2 += 1.0) { 
                for (var3 = 0; var3 < 0.01; var3 += 1.0) { 
                    for (var4 = 0; var4 < 0.01; var4 += 1.0) { 
                        //Alpha1 = var1;
                        //Alpha2 = var2;
                        //Alpha3 = var3;
                        //Delta = var4;
                        System.out.printf(" %1d %1.2f %1.2f %1.2f %1.2f ", run++, var1, var2, var3, var4);

                        // Run model:
                        corpus.zeroDVs(text);
                        for (int subject = 0; subject < NSubjects; subject++) ezreader.run(diskWriter, display, stats, text); 
                        
                        // Calculate statistics:
                        stats.analyses(diskWriter, display, text);
                    }
                }
            }
        }
    }
}

//******************************************************************************